@@ -180,11 +180,22 @@ class Preview(Enum):
     # for https://github.com/psf/black/issues/3117 to be fixed.
     string_processing = auto()
     parenthesize_conditional_expressions = auto()
+    parenthesize_long_type_hints = auto()
+    respect_magic_trailing_comma_in_return_type = auto()
     skip_magic_trailing_comma_in_subscript = auto()
     wrap_long_dict_values_in_parens = auto()
     wrap_multiple_context_managers_in_parens = auto()
     dummy_implementations = auto()
     walrus_subscript = auto()
+    module_docstring_newlines = auto()
+    accept_raw_docstrings = auto()
+    fix_power_op_line_length = auto()
+    hug_parens_with_braces_and_square_brackets = auto()
+    allow_empty_first_line_in_block = auto()
+    single_line_format_skip_with_multiple_comments = auto()
+    long_case_block_line_splitting = auto()
+    allow_form_feeds = auto()
+    respect_east_asian_width = auto()
 
 
 class Deprecated(UserWarning):
